from __future__ import annotations

from typing import Generic, TypeVar

from architectonics.core.result.error_message import ErrorMessage

TValue = TypeVar("TValue")
TValidationErrors = TypeVar("TValidationErrors")


class ServiceResult(
    Generic[
        TValue,
        TValidationErrors,
    ],
):
    __slots__ = (
        "_value",
        "_validation_errors",
        "_error_message",
    )

    def __init__(
        self,
        value: TValue | None = None,
        validation_errors: TValidationErrors | None = None,
        error_message: ErrorMessage | None = None,
    ) -> None:
        self._value = value
        self._validation_errors = validation_errors
        self._error_message = error_message

    @property
    def value(self) -> TValue | None:
        return self._value

    @property
    def validation_errors(self) -> TValidationErrors | None:
        return self._validation_errors

    @property
    def error_message(self) -> ErrorMessage | None:
        return self._error_message

    @property
    def is_validation_error(self) -> bool:
        return self._validation_errors is not None

    @property
    def is_success(self) -> bool:
        return self._value is not None and self._validation_errors is None and self._error_message is None

    @classmethod
    def success(cls, value: TValue) -> ServiceResult[TValue, TValidationErrors]:
        return cls(
            value=value,
        )

    @classmethod
    def validation_failure(cls, validation_errors: TValidationErrors) -> ServiceResult[TValue, TValidationErrors]:
        return cls(
            validation_errors=validation_errors,
        )

    @classmethod
    def failure(cls, error_message: str) -> ServiceResult[TValue, TValidationErrors]:
        return cls(
            error_message=ErrorMessage(
                message=error_message,
            ),
        )



import os

from dotenv import load_dotenv
from pydantic import BaseModel

load_dotenv()


class BaseRabbitMQSettings(BaseModel):
    RABBITMQ_USER: str = os.getenv("RABBITMQ_USER")
    RABBITMQ_PASSWORD: str = os.getenv("RABBITMQ_PASSWORD")
    RABBITMQ_HOST: str = os.getenv("RABBITMQ_HOST")
    RABBITMQ_PORT: int = int(os.getenv("RABBITMQ_PORT"))
    RABBITMQ_VHOST: str = os.getenv("RABBITMQ_VHOST")



from pydantic import BaseModel


class BaseModelCreateSchema(BaseModel):
    pass


class BaseModelCreateErrorsSchema(BaseModel):
    pass


class BaseModelGetSchema(BaseModel):
    pass


class BaseModelUpdateSchema(BaseModel):
    pass


class BaseModelUpdateErrorsSchema(BaseModel):
    pass



from pydantic import BaseModel, Field


class ObjectNotFoundResponseSchema(BaseModel):
    detail: str = Field()


class ObjectAlreadyExistsResponseSchema(BaseModel):
    detail: str = Field()


class ObjectDeletedResponseSchema(BaseModel):
    detail: str = Field()


class NotImplementedResponseSchema(BaseModel):
    detail: str = Field()


class UnauthorizedResponseSchema(BaseModel):
    detail: str = Field()



from aio_pika import ExchangeType, connect_robust

from architectonics.rabbit_mq.config.rabbit_mq_settings import BaseRabbitMQSettings


class BaseRabbitMQConsumer:
    _settings = BaseRabbitMQSettings()

    _exchange_name: str = NotImplemented
    _exchange_type: ExchangeType = NotImplemented
    _queue_name: str = NotImplemented
    _routing_key: str = NotImplemented

    _prefetch_count: int = 1

    async def connect(self):
        self.connection = await connect_robust(
            host=self._settings.RABBITMQ_HOST,
            port=self._settings.RABBITMQ_PORT,
            login=self._settings.RABBITMQ_USER,
            password=self._settings.RABBITMQ_PASSWORD,
            virtualhost=self._settings.RABBITMQ_VHOST,
        )

        self.channel = await self.connection.channel()
        await self.channel.set_qos(
            prefetch_count=self._prefetch_count,
        )

        self.exchange = await self.channel.declare_exchange(
            name=self._exchange_name,
            type=self._exchange_type,
            durable=True,
        )

        self.queue = await self.channel.declare_queue(
            name=self._queue_name,
            durable=True,
        )

        await self.queue.bind(
            exchange=self.exchange,
            routing_key=self._routing_key,
        )

    async def consume(self):
        async with self.connection:
            async with self.queue.iterator() as queue_iter:
                async for message in queue_iter:
                    async with message.process():
                        await self.process_message(message.body)

    async def process_message(self, message: bytes) -> None:

        return NotImplemented



from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from architectonics.core.config.application_settings import application_settings


class BaseApplicationFactory:
    @classmethod
    def create(cls, title) -> FastAPI:
        app = FastAPI(
            debug=application_settings.DEBUG,
            title=title,
            docs_url="/docs/",
        )

        app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )

        cls._register_app_routes(app)

        return app

    @classmethod
    def _register_app_routes(cls, app: FastAPI):
        raise NotImplementedError()



import os

from dotenv import load_dotenv
from pydantic import BaseModel

load_dotenv()


class ApplicationSettings(BaseModel):
    DEBUG: bool = os.getenv("DEBUG", "False").lower() == "true"


application_settings = ApplicationSettings()



class ObjectNotFoundException(Exception):
    """Exception raised when can't find a row."""


class ObjectAlreadyExistsException(Exception):
    """Exception raised when unique constraint failes."""


class IntegrityErrorException(Exception):
    """Exception raised when integrity constraint fails."""



from uuid import UUID

from fastapi import Depends, Header, HTTPException
from starlette.status import HTTP_400_BAD_REQUEST


def get_user_id(
    x_user_id: str = Header(
        alias="X-User-Id",
        description="Идентификатор пользователя",
    ),
) -> str:
    if not x_user_id:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="X-User-Id header is required",
        )

    try:
        _ = UUID(x_user_id, version=4)  # version=4 для проверки UUID v4
    except ValueError:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="not_valid_uuid",
        )

    return x_user_id


def get_user_roles(
    x_user_roles: str = Header(
        alias="X-User-Roles",
        description="Роли пользователя",
    ),
) -> str:
    if not x_user_roles:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="X-User-Roles header is required",
        )

    return x_user_roles.split(",")


class UserAuthService:
    def __init__(
        self,
        user_id: str = Depends(get_user_id),
        user_roles: list[str] = Depends(get_user_roles),
    ):
        self.user_id = user_id
        self.user_roles = user_roles



from abc import ABC
from typing import Callable

from asyncpg.exceptions import ForeignKeyViolationError
from sqlalchemy import delete, select, update
from sqlalchemy.exc import IntegrityError
from sqlalchemy.ext.asyncio import AsyncSession

from architectonics.core.models.base_model import BaseModel
from architectonics.infrastructure.entities.base_entity import BaseEntity
from architectonics.infrastructure.repositories.base_exceptions import (
    IntegrityErrorException,
    ObjectAlreadyExistsException,
    ObjectNotFoundException,
)


class BaseRepository(ABC):
    _entity: type[BaseEntity] = NotImplemented
    _session: Callable = NotImplemented
    _integrity_error: type[IntegrityError] = IntegrityError

    def get_session(self) -> AsyncSession:
        return self._session()

    @property
    def model_fields(self):
        return self._entity.__table__.columns

    async def create_model(
        self,
        model: BaseModel,
    ) -> BaseModel:

        entity: BaseEntity = self._entity.from_model(
            model=model,
        )

        async with self.get_session() as session:
            session.add(entity)

            try:
                await session.commit()
            except self._integrity_error as e:
                raise ObjectAlreadyExistsException(e)

            return entity.to_model()

    async def get_model_by_id(
        self,
        model_id: str,
    ) -> BaseModel | None:

        statement = select(
            self._entity,
        ).where(
            self._entity.id == model_id,
        )

        async with self.get_session() as session:

            result = await session.execute(
                statement=statement,
            )

            entity = result.scalars().first()

            if entity is None:
                return None

            return entity.to_model()

    async def update_model(
        self,
        model: BaseModel,
    ) -> BaseModel:

        updated_values = model.to_update_dict()

        update_statement = (
            update(
                self._entity,
            )
            .where(
                self._entity.id == model.id,
            )
            .values(
                **updated_values,
            )
        )

        async with self.get_session() as session:
            try:
                result = await session.execute(update_statement)

                await session.commit()
            except self._integrity_error as e:
                orig = getattr(e.orig, "__cause__", None)

                if isinstance(orig, ForeignKeyViolationError):
                    raise ObjectNotFoundException()

                raise IntegrityErrorException(e)

            return model

    async def delete_model(
        self,
        model_id: str,
    ) -> None:

        statement = delete(
            self._entity,
        ).where(
            self._entity.id == model_id,
        )

        async with self.get_session() as session:
            result = await session.execute(
                statement=statement,
            )

            if result.rowcount == 0:
                raise ObjectNotFoundException()

            await session.commit()

    async def get_models_list(
        self,
    ) -> list[BaseModel]:

        statement = select(
            self._entity,
        )

        async with self.get_session() as session:

            result = await session.execute(
                statement=statement,
            )

            models = result.scalars().all()

            return [model.to_model() for model in models]



import uuid

from sqlalchemy import Column, DateTime, String
from sqlalchemy.orm import as_declarative

from architectonics.common.utils.utils import get_current_datetime
from architectonics.core.models.base_model import BaseModel


@as_declarative()
class BaseEntity:
    __abstract__ = True

    id = Column(
        String,
        primary_key=True,
        default=lambda: str(uuid.uuid4()),
    )

    created_at = Column(
        DateTime,
        default=get_current_datetime,
    )

    updated_at = Column(
        DateTime,
        default=get_current_datetime,
        onupdate=get_current_datetime,
    )

    PK_FIELD = "id"

    def to_model(self) -> BaseModel:
        raise NotImplementedError()

    @classmethod
    def from_model(model: BaseModel) -> BaseEntity:
        raise NotImplementedError()



from architectonics.common.enums.step_type import StepType
from pydantic import BaseModel


class StepModelContract(BaseModel):
    id: str
    title: str | None
    icon: str | None
    is_publish: bool
    step_type: StepType



from dataclasses import asdict, dataclass, field
from datetime import datetime
import uuid


@dataclass(
    kw_only=True,
)
class BaseModel:
    id: str = field(
        default_factory=lambda: str(uuid.uuid4()),
    )

    created_at: datetime = field(
        default_factory=datetime.utcnow,
    )

    updated_at: datetime = field(
        default_factory=datetime.utcnow,
    )

    def to_update_dict(self) -> dict[str, any]:
        now = datetime.utcnow()

        values = {k: v for k, v in asdict(self).items() if k not in {"id", "created_at"} and v is not None}

        values["updated_at"] = now
        self.updated_at = now

        return values



from dataclasses import dataclass, fields


@dataclass
class BaseModelValidationErrors:

    def has_errors(self) -> bool:
        for f in fields(self):
            value = getattr(self, f.name)

            if isinstance(value, list) and len(value) > 0:
                return True

        return False



from starlette.status import (
    HTTP_200_OK,
    HTTP_404_NOT_FOUND,
    HTTP_409_CONFLICT,
    HTTP_422_UNPROCESSABLE_CONTENT,
)

from architectonics.core.models.base_model import BaseModel
from architectonics.core.models.deleted_models import DeletedModel
from architectonics.core.result.service_result import ServiceResult
from architectonics.core.services.base_schemas import (
    BaseModelCreateSchema,
    BaseModelUpdateSchema,
)
from architectonics.core.validation.base_model_validation_errors import BaseModelValidationErrors
from architectonics.infrastructure.repositories.base_exceptions import (
    IntegrityErrorException,
    ObjectAlreadyExistsException,
    ObjectNotFoundException,
)
from architectonics.infrastructure.repositories.base_repository import BaseRepository


class BaseService:
    _repository: BaseRepository = NotImplemented

    async def get_models_list(
        self,
    ) -> ServiceResult[list[BaseModel], BaseModelValidationErrors]:

        models = await self._repository.get_models_list()

        return ServiceResult[list[BaseModel], BaseModelValidationErrors].success(models)

    async def validate(self, model: BaseModel) -> BaseModelValidationErrors:

        errors = BaseModelValidationErrors()

        return errors

    async def create_model(
        self,
        model: BaseModel,
    ) -> ServiceResult[BaseModel, BaseModelValidationErrors]:

        validation_errors = await self.validate(model)

        if validation_errors.has_errors():
            return ServiceResult[BaseModel, BaseModelValidationErrors].validation_failure(
                validation_errors=validation_errors,
            )

        try:
            model = await self._repository.create_model(
                model=model,
            )
        except ObjectAlreadyExistsException:
            return ServiceResult[BaseModel, BaseModelValidationErrors].failure(
                error_message="object_already_exists",
            )

        return ServiceResult[BaseModel, BaseModelValidationErrors].success(
            value=model,
        )

    async def get_model_by_id(
        self,
        model_id: str,
    ) -> ServiceResult[BaseModel, BaseModelValidationErrors]:

        model = await self._repository.get_model_by_id(
            model_id=model_id,
        )

        if model is None:
            return ServiceResult[BaseModel, BaseModelValidationErrors].failure(
                error_message="object_not_found",
            )

        return ServiceResult[BaseModel, BaseModelValidationErrors].success(
            value=model,
        )

    async def update_model(
        self,
        model: BaseModel,
    ) -> ServiceResult[BaseModel, BaseModelValidationErrors]:

        validation_errors = await self.validate(model)

        if validation_errors.has_errors():
            return ServiceResult[BaseModel, BaseModelValidationErrors].validation_failure(
                validation_errors=validation_errors,
            )

        try:
            model = await self._repository.update_model(
                model=model,
            )
        except IntegrityErrorException as error:
            return ServiceResult[BaseModel, BaseModelValidationErrors].failure(error_message=error)
        except ObjectNotFoundException:
            return ServiceResult[BaseModel, BaseModelValidationErrors].failure(
                error_message="object_not_found",
            )

        return ServiceResult[BaseModel, BaseModelValidationErrors].success(value=model)

    async def delete_model(
        self,
        model_id: str,
    ) -> ServiceResult[DeletedModel, BaseModelValidationErrors]:

        try:
            await self._repository.delete_model(
                model_id=model_id,
            )
        except ObjectNotFoundException:
            return ServiceResult[BaseModel, BaseModelValidationErrors].failure(
                error_message="object_not_found",
            )

        return ServiceResult[DeletedModel, BaseModelValidationErrors].success(
            value=DeletedModel(
                id=model_id,
            ),
        )



import os

from dotenv import load_dotenv
from pydantic import BaseModel

load_dotenv()


class DatabaseSettings(BaseModel):
    DATABASE_PREFIX: str = "postgresql+asyncpg"

    POSTGRES_USER: str = os.getenv("POSTGRES_USER")
    POSTGRES_PASSWORD: str = os.getenv("POSTGRES_PASSWORD")
    POSTGRES_HOST: str = os.getenv("POSTGRES_HOST")
    POSTGRES_PORT: int = os.getenv("POSTGRES_PORT")
    POSTGRES_DATABASE_NAME: str = os.getenv("POSTGRES_DATABASE_NAME")

    DATABASE_CONNECTION_STRING: str = (
        f"{DATABASE_PREFIX}://{POSTGRES_USER}:" f"{POSTGRES_PASSWORD}@{POSTGRES_HOST}:{POSTGRES_PORT}/{POSTGRES_DATABASE_NAME}"
    )


database_settings = DatabaseSettings()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['architectonics',
 'architectonics.common',
 'architectonics.common.consts',
 'architectonics.common.enums',
 'architectonics.common.utils',
 'architectonics.contracts',
 'architectonics.contracts.action_type',
 'architectonics.contracts.step_model',
 'architectonics.core',
 'architectonics.core.config',
 'architectonics.core.factory',
 'architectonics.core.models',
 'architectonics.core.result',
 'architectonics.core.schemas',
 'architectonics.core.services',
 'architectonics.core.validation',
 'architectonics.infrastructure',
 'architectonics.infrastructure.config',
 'architectonics.infrastructure.entities',
 'architectonics.infrastructure.repositories',
 'architectonics.rabbit_mq',
 'architectonics.rabbit_mq.config',
 'architectonics.rabbit_mq.consumer']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy==2.0.46',
 'aio-pika==9.5.8',
 'alembic==1.18.1',
 'asyncpg==0.31.0',
 'dotenv==0.9.9',
 'fastapi==0.128.0',
 'pydantic==2.12.5',
 'uvicorn==0.40.0']

setup_kwargs = {
    'name': 'architectonics',
    'version': '0.0.42',
    'description': '',
    'long_description': None,
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.14,<4.0',
}


setup(**setup_kwargs)



from sqlalchemy.ext.asyncio import AsyncEngine, AsyncSession, create_async_engine
from sqlalchemy.orm import sessionmaker

from architectonics.infrastructure.config.database_settings import database_settings


def get_engine() -> AsyncEngine:
    return create_async_engine(
        database_settings.DATABASE_CONNECTION_STRING,
        future=True,
        echo=False,
    )


def get_session(engine: AsyncEngine) -> sessionmaker:
    return sessionmaker(
        bind=engine,
        class_=AsyncSession,
        expire_on_commit=False,
    )
